<?php

if (!defined('BASEPATH'))
    exit('No direct script access allowed');

require_once(FCPATH . '/application/libraries/base_table.php');

class Client_table extends Base_table {

	function __construct()
	{
		parent::__construct(array('tbl'=>'client', 'tbl_key'=>'client_id'));
    }
	
	public function formatdata(&$data) 
	{
		if (!empty($data['client_code'])) {
			$data['client_code'] = substr($data['client_code'],0,6).' - ';
		}

        return true;
    }
 
    function save(&$data)
    {
		$result = array();
    	$values = $data;

		$client_id = !empty($data[$this->tbl_key])?$data[$this->tbl_key]:0;

		if (!license_limit($client_id, 'clients')) {
			$this->CI->response->error(sprintf(lang('UPGRADE_YOUR_VERSION'),'clients'));
			return false;
		}

		if ($client_id = parent::save($values))
		{
			// save contacts
            if (!empty($data['contacts']))
            {
            	foreach ($data['contacts'] as $key => $contact)
            	{
            		$contact['client_id']	= $client_id;
            		$data['contacts'][$key] = $this->save_contact($contact);
            	}
            }

            //delete contacts
			if (!empty($data['deleted_contacts']))
            {
            	$this->delete_seleted_contact($data['deleted_contacts']);
            }
            return $client_id;
        }
        return false;
    }
    
    function save_contact ($data)
    {
		$user_id	= $this->CI->ot_user->get('user');

		// store address
		$values				= $data;
		$values['tbl_key']	= 'address_id';
		$values['tbl']		= 'address';
		
		if ($add_id = parent::save($values))
		{
			$data['address_id'] = $add_id;
			return $this->save_client_contact($data);
		}

		return false;
    }
    
	function save_client_contact ($values)
    {
		$values['tbl_key']	= 'client_contact_id';
		$values['tbl']		= 'client_contact';
		
		if ($id = parent::save($values))
		{
			return true;
		}

		return false;
    }   
 
	public function delete($client_id)
	{
		$this->CI->db->select('address_id');
		$this->CI->db->from('client_contact');
		$this->CI->db->where('client_id',$client_id);
		$query = $this->CI->db->get();
		$addr_ids = array();
		
		// delete address detail	
		if ($query->num_rows() > 0)
		{
		   foreach ($query->result() as $row)
		   {
		      $addr_ids[] =  $row->address_id;
		   }
		   $this->CI->db->where_in('address_id', $addr_ids);
		   $this->CI->db->delete('address');
		} 
		// delete client contanct
		$this->CI->db->where(array('entity_id' => $this->entity_id, 'client_id' => $client_id));
		$this->CI->db->delete('client_contact');
		
		//delete client
		$this->CI->db->where(array('entity_id' => $this->entity_id, 'client_id' => $client_id));
		$this->CI->db->delete('client');
		
		return true;
	}
	
	public function delete_seleted_contact ($contact_ids)
	{
		$this->CI->db->select('address_id');
		$this->CI->db->from('client_contact');
		$this->CI->db->where(array('entity_id' => $this->entity_id));
		$this->CI->db->where_in('client_contact_id',$contact_ids);
		$query = $this->CI->db->get();
		$addr_ids = array();
		
		// delete address detail	
		if ($query->num_rows() > 0)
		{
		   foreach ($query->result() as $row)
		   {
		      $addr_ids[] =  $row->address_id;
		   }
		   $this->CI->db->where(array('entity_id' => $this->entity_id));
		   $this->CI->db->where_in('address_id', $addr_ids);
		   $this->CI->db->delete('address');
		} 
		// delete client contanct
		$this->CI->db->where(array('entity_id' => $this->entity_id));
		$this->CI->db->where_in('client_contact_id',$contact_ids);
		$this->CI->db->delete('client_contact');
		
		return true;
	}
}
