<?php if ( ! defined('BASEPATH')) exit('No direct script access allowed');

require_once(APPPATH . '/models/projects/mod.projects_member.php');

class Projects_pm extends Projects_member
{
	function __construct()
	{
		parent::__construct();
		$this->_entity_id	= $this->ot_app->get('entity_id');
		$this->_user_id		= $this->ot_user->get('user_id');
	}

	public function search_members($term="")
	{
		/*if (empty($term))
		{
			return false;
		}*/

		$this->db->select('e.user_id, e.first_name, e.user_role, e.user_image');
		$this->db->from('entity_users as e');
		$this->db->where('e.entity_id =', $this->ot_app->get('entity_id'));
		if (!empty($term))
		$this->db->like('e.first_name', $term);
		$this->db->order_by('e.first_name');
		$query	= $this->db->get();

		return $query->result_array();
	}

	public function save(&$data)
 	{
 		$this->load->library('project_table');

 		$project_id	= $this->project_table->save($data);
 		if (!$project_id)
 		{
 			return false;
 		}

		return $project_id;
	}

	public function team_members($roles=null, $all=false)
	{
		$this->db->select('e.user_id, e.first_name, e.last_name, e.user_role, e.user_image');
		$this->db->from('entity_users as e');
		$this->db->where('e.entity_id =', $this->ot_app->get('entity_id'));
		if (!$all)
		{
			$this->db->where('e.is_active', '1');
		}
		/*$this->db->where_in('user_role', array_keys($roles));*/
		$this->db->order_by('e.first_name');
		$query	= $this->db->get();

		return $query->result_array();
	}

	public function map_attachment_to_project ($project_id, $attachments)
	{

		$attahment_ids = array();
		foreach ($attachments as $att)
		{
			$attahment_ids[] = $att['attachment_id'];
		}
		$this->db->where(array('module_type' => 'P', 'module_id' => 0));
		$this->db->where_in('attachment_id', $attahment_ids);
		$this->db->update('attachment', array('module_id' => $project_id));

		return true;
	}

	public function save_new($data)
	{
		$this->load->library('project_table');

 		$project_id	= $this->project_table->save($data['project']);
 		if (!$project_id)
 		{
 			return false;
 		} 		

		if (!empty($data['members']))
 		{
			if (!$this->project->store_project_members($project_id, $data['members']))
	 			return false;
 		} else if (empty($data['project']['project_id'])){
			$this->load->library('project_users_table');
			if (! $this->project_users_table->store_default_data($project_id))
			{
					return false;
			}
		}

		if (!empty($data['task_types']) && !$this->project->store_project_task_types($project_id, $data['task_types']))
 		{
 			return false;
 		}

		if (!empty($data['document']) && !empty($data['document']['attachment_details']) && !$this->map_attachment_to_project($project_id, $data['document']['attachment_details']))
 		{
 			return false;
 		}

 		return $project_id;
	}

	public function get_project_count()
	{
		$this->db->select('SUM(pr.is_active=1) AS active_count, SUM(pr.is_active=0) AS archive_count', false);
		$this->db->from('project as pr');
		$this->db->join('project_users as pru', 'pr.entity_id = pru.entity_id and pr.project_id=pru.project_id', 'INNER');

		$this->db->where(array(
			'pr.entity_id' => $this->ot_app->get('entity_id'),
			'pru.user_id'	=> $this->ot_user->get('user_id')
		));

		$query = $this->db->get();

		$result = $query->row_array();

		return $result;
	}

	public function projects_list_query($select, $options = array())
	{
		$restricted_fields = array('pr.budget_value', 'pr.actual_value', 'pr.budget_type');

		$split = explode(',', $select);

		foreach($split as $key => $val)
		{
			$t_val = $val = trim($val);
			
			if ($val && in_array($val,$restricted_fields))
			{
				$value = explode('.',$val);
				$val = (isset($value[1]))?$value[1]:$val;
				$split[$key] = '-1 AS '.$val;
				if ($val=='actual_value')
					$split[$key] = 'IF((pru.user_role="P" AND (pr.budget_type="HA" OR pr.budget_type="HT" OR pr.budget_type="HP")), '.$t_val.', 0) AS '.$val;
			}
		}
		$select = implode(',', $split);
		$where	= array();

		$this->db->select($select.", pru.user_role", false);
		$this->db->from('project as pr');
		$this->db->join('project_users as pru', 'pr.entity_id = pru.entity_id and pr.project_id=pru.project_id', 'INNER');
		$this->db->join('client as cl', 'pr.entity_id=cl.entity_id and pr.client_id = cl.client_id', 'INNER');

		if (!empty($options['filters']))
		{
			foreach($options['filters'] as $key=>$val)
			{
				if (is_array($val))
					$this->db->where_in($key,$val);
				else
					$where[$key] = $val;
			}
		}

		$where['pr.entity_id']	= $this->_entity_id;
		if (empty($where['pru.user_id']))
		{
			$where['pru.user_id']	= $this->_user_id;
		}

		$this->db->where($where);

		if (empty($options['union_query']))
		{
			$order_by 			= !empty($options['order_by']) ? $options['order_by'] : 'pr.project_name';
			$order_by_direction = !empty($options['order_by_direction']) ? $options['order_by_direction'] : 'asc';

			$this->db->order_by($order_by, $order_by_direction);
		}

		$from = !empty($options['from'])?$options['from']:0;
		if (!empty($options['count']))
		{
			$this->db->limit($options['count'], $from);
		}

		return $this->db->get();
	}

	// return projects in as array of standard array indexes
	public function get_projects($select = '', $options = array())
	{
		$select = ($select)?$select:'pr.project_id, cl.client_id, concat(pr.project_code, pr.project_name) AS project_name, pr.project_code,  pr.members_count, pr.created_date, pr.is_active, pr.bg_color,pr.actual_value, pr.created_by';
		$query = $this->projects_list_query($select, $options);

		return $query->result_array();
	}
}
