<?php if ( ! defined('BASEPATH')) exit('No direct script access allowed');

class Setup extends CI_Controller {

	// PHP Version
	private $version	= '5.4.1';

	private $isDependent	= 0;

	private $database_php	= '';

	private $config_php	= '';

	private $lock_file	= '';

	private $installer_file	= '';

	private $database_file	= '';

	private $message	= '';

	private $dbConn	= null;

	private $download_url	= 'http://www.outputtime.com/releases/index.php';

// 	private $email_php	= '';

	private $extensions_needed	= array(
		'zlib'	=> array(
			'name'	=> 'zlib',
			'value'	=> false,
			'label'	=> 'Zlib Compression plugin (zlib)'
		),
		'zip'	=> array(
			'name'	=> 'zip',
			'value'	=> false,
			'label'	=> 'Zip Compression plugin (zip)'
		),
		'xml'	=> array(
			'name'	=> 'xml',
			'value'	=> false,
			'label'	=> 'XML plugin (xml)'
		),
		'mysql'	=> array(
			'name'	=> 'mysql',
			'value'	=> false,
			'label'	=> 'MySql plugin (mysql)'
		),
// 		'mysqli'	=> array(
// 			'name'	=> 'mysqli',
// 			'value'	=> false,
// 			'label'	=> 'MySqli plugin (mysqli)'
// 		),
		'json'	=> array(
			'name'	=> 'json',
			'value'	=> false,
			'label'	=> 'JSON plugin (json)'
		),
		'curl'	=> array(
			'name'	=> 'curl',
			'value'	=> false,
			'label'	=> 'CURL plugin (curl)'
		),
		'sockets'	=> array(
			'name'	=> 'sockets',
			'value'	=> false,
			'label'	=> 'PHP Socket (sockets)'
		),
		'otime'	=> array(
			'name'	=> 'otime',
			'value'	=> false,
			'label'	=> 'Outputtime plugin (otime)',
			'path'	=> 'http://www.outputtime.com/installer/'
		),
// 		array(
// 			'name'	=> 'gd',
// 			'value'	=> false,
// 			'label'	=> 'GD Library'
// 		),
	);

	private $installed	= false;

	function __construct()
	{
		$this->database_php	= FCPATH . OT_APPPATH . 'config/client.db.php';
		$this->config_php	= FCPATH . OT_APPPATH . 'config/client.php';
		$this->lock_file	= FCPATH . 'tmp/setup.lock';
		$this->database_file	= FCPATH . "database/outputtime_dump.sql";
		$this->installer_file	= FCPATH . 'installer.php';
// 		$this->email_php	= FCPATH . OT_APPPATH . 'config/email.php';

		parent::__construct();

		if ($this->is_locked())
		{
			return false;
		}

		if ($this->is_already_installed())
		{
			$this->installed	= true;
			return false;
		}
	}

	public function index()
	{
		$data	= array();

		$data['installed']	= false;
		$data['isDependent']	= false;

		if ($this->installed)
		{
			$data['installed']	= true;
			$this->load->view('setup/index', $data);
			return true;
		}

		// PHP Version >= 5.3.1
		// config folder Writable

		$this->version	= '5.3.1';

		$data['extensions']	= $this->check_extensions();

		$data['php_version']	= array(
			'name'	=> 'version',
			'value'	=> false,
			'label'	=> 'PHP Version ' . PHP_VERSION . ' >= ' . $this->version
		);

		if (version_compare(PHP_VERSION, $this->version) >= 0)
		{
			$this->isDependent++;
			$data['php_version']['value']	= true;
		}

		$data['config']	= array(
			'name'	=> 'config',
			'value'	=> false,
			'label'	=> 'Config folder Writable'
		);

		if (is_writable(dirname($this->database_php)))
		{
			$this->isDependent++;
			$data['config']['value']	= true;
		}

		$data['isDependent']	= ($this->isDependent == (count($data['extensions']) + 2)) ? false : true;

// 		$version	= explode('.', PHP_VERSION);
// 		$bit	= array('4'	=> '32', '8'	=> '64');
// 		$file	= PHP_SHLIB_SUFFIX === 'dll' ? 'php_' : '';
// 		$file	.= 'otime_' . $version[0] . '.' . $version[1] . '_' . $bit[PHP_INT_SIZE] . '.' . PHP_SHLIB_SUFFIX;
/*
		$url	= $this->download_url . '?extension=1';
		$url	.= '&bit=' . (PHP_INT_SIZE * 8);
		$url	.= '&version=' . $version[0] . '.' . $version[1];
		$url	.= '&type=' . PHP_SHLIB_SUFFIX;
*/
		$url	= site_url('setup/extensions');

		$data['extensions']['otime']['url']	= $url;
		$data['extensions']['otime']['dislay_file_name']	= (PHP_SHLIB_SUFFIX === "dll" ? "php_otime." : "otime.") . PHP_SHLIB_SUFFIX;

		if ($dbconfig = $this->can_connect_db())
		{
			$data['dbconfig']	= $dbconfig;
		}

		$this->load->view('setup/index', $data);

		return true;
	}

	private function can_connect_db()
	{
		$file	= FCPATH . 'conf.ini';
		if (file_exists($file))
		{
			$ini_data	= parse_ini_file($file, true);

			if (empty($ini_data) || empty($ini_data['database']))
			{
				return false;
			}

			$data	= $ini_data['database'];
			$data['dbdriver']	= "mysql";
			$data['dbprefix']	= "";
			$data['pconnect']	= false;

			if ($this->is_db_exists($data))
			{
				return $data;
			}
		}

		return false;
	}

	public function go()
	{
		if ($this->is_already_installed() && file_exists($this->installer_file))
		{
			rename($this->installer_file, $this->installer_file . "_");
		}
		redirect('/');
	}

	public function extensions()
	{
		$extensions_dir	= FCPATH . APPPATH . 'extensions/';

		$version	= explode('.', PHP_VERSION);
		$version	= $version[0] . '.' . $version[1];
		$bit		= (PHP_INT_SIZE * 8);
		$type		= PHP_SHLIB_SUFFIX;

		if ($type === "dll")
		{
			$file	= "php_otime_" . $version . "_" . $bit . 'bit.' . $type;
			$download_name	= "php_otime." . $type;
		}
		else
		{
			$file	= "otime_" . $version . "_" . $bit . "bit." . $type;
			$download_name	= "otime." . $type;
		}

		$this->download_file($extensions_dir . $file, $download_name, true);
	}

	private function download_file($file='', $file_name='', $isBinary=false)
	{
		if (file_exists($file))
		{
			if ($isBinary)
			{
				header("Content-type: application/octet-stream");
			}
			else
			{
				header("Content-type: application/zip");
			}
			header("Content-Disposition: attachment; filename=" . $file_name);
			header("Content-length: " . filesize($file));
			header("Pragma: no-cache");
			header("Expires: 0");
			readfile($file);
		}
		else
		{
			header($_SERVER["SERVER_PROTOCOL"] . " 404 Not Found");
		}
	}

	private function write_output($success=true, $message="", $data=array())
	{
// 		header("Content-type: text/json; charset=utf-8");
		header("Content-type: application/json; charset=utf-8");
		echo json_encode(array($success, $message, $data));
		exit;
	}

	private function is_already_installed()
	{
		if (file_exists($this->database_php) && file_exists($this->config_php))
		{
			return true;
		}

		return false;
	}

	private function remove_config()
	{
		@unlink($this->database_php);
		@unlink($this->config_php);
		return true;
	}

	private function check_extensions()
	{
		$arr	= $this->extensions_needed;
		foreach($arr as $key=>$val)
		{
			if (extension_loaded($val['name']))
			{
				$arr[$key]['value']	= true;
				$this->isDependent++;
			}
		}

		return $arr;
	}

	private function create_lock()
	{
		@file_put_contents($this->lock_file, '1');
	}

	private function remove_lock()
	{
		@unlink($this->lock_file);
	}

	private function is_locked()
	{
		if(file_exists($this->lock_file))
		{
			return true;
		}

		return false;
	}

	public function check_license()
	{
		$data	= $_POST;

		if (!empty($data) && $data['license_key'])
		{
			if (!$lic_data=$this->decrypt($data['license_key']))
			{
				$this->write_output(false, 'Invalid License Key.', array());
				return false;
			}
			$this->write_output(true, '', array('license_detail'=>$lic_data));
			return true;
		}

		$this->write_output(false, 'License key missing.', array());
		return false;
	}

	private function escape_str($str='')
	{
		return mysql_real_escape_string($str);
	}
	public function test_email()
	{
		$data	= $_POST;

		if (!empty($data['smtp_config']) && !empty($data['to_email']) && !empty($data['license_key']))
		{
			if (!$lic_data=$this->decrypt($data['license_key']))
			{
				$this->write_output(false, 'Invalid License Key.', array());
				return false;
			}

			$smtp	= $data['smtp_config'];

			$config = Array (
				'protocol' => 'smtp',
				'smtp_host' => $smtp['smtp_host'],
				'smtp_port' => $smtp['smtp_port'],
				'smtp_user' => $smtp['smtp_user'],
				'smtp_pass' => $smtp['smtp_pass'],
				'charset' => "iso-8859-1",
				'mailtype' => 'html',
				'smtp_timeout' => 60,
				'newline'	=> "\r\n"
			);

			$this->load->library('email', $config);
			$this->email->from($smtp['smtp_user'], 'Output Time');
			$this->email->to($data['to_email']);
			$this->email->subject('SMTP Test Mail From OutputTime.');
			$this->email->message(nl2br('SMTP Test Mail From OutputTime On-Premises Server of ' . $lic_data['company_name']));

			if (!$this->email->send())
			{
				$this->write_output(false, 'Can\'t send email with this SMTP configuration.', array());
				return false;
			}

			$this->write_output(true, 'SMTP working fine.', array());
			return true;
		}

		$this->write_output(false, 'Required field missing.', array());
		return false;
	}

	public function install()
	{
		if ($this->is_locked())
		{
			$this->write_output(false, 'Already Installer running.', array());
			return;
		}

		$data	= $_POST;

		$return	= false;
		if (!empty($data['dbconfig']) && !empty($data['smtpConfig']) && !empty($data['otherDetails']))
		{
			$this->create_lock();

			if ($this->is_db_exists($data['dbconfig']))
			{
				if (!$eData=$this->decrypt($data['otherDetails']['license_key']))
				{
					$this->remove_lock();
					$this->write_output(false, 'Invalid License Key.', array());
					return;
				}

				$this->update_db_config($data['dbconfig']);
				$data['smtpConfig']['license_key']	= $data['otherDetails']['license_key'];
				$this->update_other_config($data['smtpConfig']);

				$this->init_db_conn($data['dbconfig']);
				$this->import_database();
				$this->update_domain($data['otherDetails'], $eData);

				$this->remove_lock();
				$this->write_output(true, '', array());
				return;
			}

// 			$this->remove_config();
			$this->remove_lock();
			$this->write_output(false, $this->message, array());
		}
	}

	private function update_db_config($data=array())
	{
		$this->load->helper('file');

		$data['autoinit']	= TRUE;

		$content	= "<?php  if ( ! defined('BASEPATH')) exit('No direct script access allowed');" . PHP_EOL . PHP_EOL;

		foreach ($data as $key=>$val)
		{
			if (is_numeric($val))
			{
				$content	.= "\$db[\$active_group]['" . $key . "'] = " . $val . ";" . PHP_EOL;
			}
			else if (is_bool($val))
			{
				$content	.= "\$db[\$active_group]['" . $key . "'] = " . ($val ? "TRUE" : "FALSE" ) . ";" . PHP_EOL;
			}
			else
			{
				$content	.= "\$db[\$active_group]['" . $key . "'] = '" . $val . "';" . PHP_EOL;
			}
		}

		$content	.= PHP_EOL . "?>";

		$return	= write_file($this->database_php, $content);

		@chmod($this->database_php, 0777);
		
		return $return;
	}

	private function update_other_config($data=array())
	{
		$this->load->helper('file');

		$data['enable_hooks']	= TRUE;
		$data['sess_use_database']	= TRUE;

		$content	= "<?php  if ( ! defined('BASEPATH')) exit('No direct script access allowed');" . PHP_EOL . PHP_EOL;

		foreach ($data as $key=>$val)
		{
			if ($key == 'license_key')
			{
				continue;
			}
			if (is_numeric($val))
			{
				$content	.= "\$config['" . $key . "'] = " . $val . ";" . PHP_EOL;
			}
			else if (is_bool($val))
			{
				$content	.= "\$config['" . $key . "'] = " . ($val ? "TRUE" : "FALSE" ) . ";" . PHP_EOL;
			}
			else
			{
				$content	.= "\$config['" . $key . "'] = '" . $this->escape_str($val) . "';" . PHP_EOL;
			}
		}

		// Proxy details for use in PHP CURL.
		$content	.= "\$config['use_proxy'] = FALSE;" . PHP_EOL;
		$content	.= "\$config['http_proxy'] = '';" . PHP_EOL;
		$content	.= "\$config['http_proxy_port'] = '';" . PHP_EOL;
		$content	.= "\$config['https_proxy'] = '';" . PHP_EOL;
		$content	.= "\$config['https_proxy_port'] = '';" . PHP_EOL;
		$content	.= "\$config['proxy_username'] = '';" . PHP_EOL;
		$content	.= "\$config['proxy_password'] = '';" . PHP_EOL;

		$content	.= PHP_EOL . "?>";

		$return	= write_file($this->config_php, $content);

		@chmod($this->config_php, 0777);

		return $return;
	}

	public function db_exist()
	{
		$data	= $_POST;
		if ($this->is_db_exists($data['dbconfig']))
		{
			$this->write_output(true, "", array());
		}
		else
		{
			$this->write_output(false, $this->message, array());
		}
	}

	private function is_db_exists($config=array())
	{
// 		$config['hostname'] = "localhost";
// 		$config['username'] = "myusername";
// 		$config['password'] = "mypassword";
// 		$config['database'] = "mydatabase";
// 		$config['dbdriver'] = "mysql";
// 		$config['dbprefix'] = "";
// 		$config['pconnect'] = false;
		$config['db_debug'] = false;

		$conn	= @$this->load->database($config, true);

		if (!$conn->conn_id)
		{
			$this->message	= 'Couldn\'t connet to database using given settings.';
			return false;
		}

		if (!$conn->db_select())
		{
			$this->message	= 'Database not exist.';
			return false;
		}

		$this->message	= 'Database exist.';
		return true;
	}

	private function update_domain($data=array(), $eData=array())
	{
		$this->load->library('encrypt');
		$data['password']	= $this->encrypt->sha1($data['password']);

		$current_date	= gmdate('Y-m-d H:i:s');
		$host		= parse_url(base_url(), PHP_URL_HOST);//['host'];

// 		$eData['sub_domain']	= strtolower(str_replace(" ", "-", $eData['company_name']));
// 		$domains	= implode(array($host, str_replace('www.', $eData['sub_domain'] . '.', $host)), ",");
		$eData['sub_domain']	= $host;
		$domains	= $host;

// 		if (filter_var($host, FILTER_VALIDATE_IP))
// 		{
// 			$domains	= $host;
// 		}

		$timezoneCode	= $data['time_zone'];

		//================================================================================
		$dbData	= array(
			'entity_id'		=> $eData['entity_id'],
			'name'			=> $eData['company_name'],
			'domain_name'	=> $eData['sub_domain'],
// 			'domain_name'	=> '',
			'created_date'	=> $current_date
		);
		$this->dbConn->where('entity_id', '1');
		$this->dbConn->update('entity', $dbData);
		//================================================================================
		$dbData	= array(
			'entity_id'		=> $eData['entity_id'],
			'company_name'	=> $eData['company_name'],
			'domain_url'	=> $domains,
			'user_name'		=> $data['first_name'],
			'register_date'	=> $current_date,
			'email'			=> $data['email'],
			'password'		=> $data['password'],
			'max_users'		=> '',//$eData['max_users'],
			'exp_date'		=> '',//$eData['expiry_date'],
			'plan'			=> '',//$eData['plan'],
			'license_key'	=> $data['license_key']
		);
		$this->dbConn->where('entity_id', '1');
		$this->dbConn->update('entity_accounts', $dbData);
		//================================================================================
		$dbData	= array(
			'entity_id'		=> $eData['entity_id'],
			'email'			=> $data['email'],
			'password'		=> $data['password'],
			'first_name'	=> $data['first_name'],
			'last_name'		=> $data['last_name'],
			'created_date'	=> $current_date,
			'time_zone_code'=> $timezoneCode
		);
		$this->dbConn->where('entity_id', '1');
		$this->dbConn->update('entity_users', $dbData);
		//================================================================================
		$dbData	= array(
			'entity_id'		=> $eData['entity_id'],
			'created_date'	=> $current_date,
			'time_zone_code'	=> $timezoneCode
		);

		$this->dbConn->where('entity_id', '1');
		$this->dbConn->update('entity_setting', $dbData);

		//================================================================================

		$dbData	= array(
			'entity_id'		=> $eData['entity_id'],
			'created_date'	=> $current_date
		);
		$this->dbConn->where('entity_id', '1');
		$this->dbConn->update('expense_type', $dbData);
		//================================================================================
		$dbData	= array('entity_id' => $eData['entity_id']);

		$this->dbConn->where('entity_id', '1');
		$this->dbConn->update('entity_daysoff', $dbData);

		$this->dbConn->where('entity_id', '1');
		$this->dbConn->update('task_status', $dbData);

		$this->dbConn->where('entity_id', '1');
		$this->dbConn->update('task_type', $dbData);

		$this->dbConn->where('entity_id', '1');
		$this->dbConn->update('user_department', $dbData);

		return true;
	}

	private function decrypt($string="")
	{
		$this->load->library('ot_license');
		return $this->ot_license->get_details($string);
	}

	private function is_valid_license($license="")
	{
		if (empty($license))
		{
			return false;
		}

		return true;
	}

	private function init_db_conn($config=array())
	{
		if (!$this->dbConn)
		{
			$this->dbConn	= $this->load->database(array_merge($config, array('dbprefix'=>'ot_')), true);
		}
	}

	private function import_database()
	{
		$this->load->helper('file');

		$queries	= $this->splitSql(read_file($this->database_file));

		foreach ($queries as $key=>$val)
		{
			$this->dbConn->query($val);
		}

// 		@unlink($this->database_file);

		return true;
	}

	private function splitSql($sql)
	{
		$start = 0;
		$open = false;
		$char = '';
		$end = strlen($sql);
		$queries = array();
	
		for ($i = 0; $i < $end; $i++)
		{
			$current = substr($sql, $i, 1);
			if (($current == '"' || $current == '\''))
			{
				$n = 2;
				while (substr($sql, $i - $n + 1, 1) == '\\' && $n < $i)
				{
					$n++;
				}
				if ($n % 2 == 0)
				{
					if ($open)
					{
						if ($current == $char)
						{
							$open = false;
							$char = '';
						}
					}
					else
					{
						$open = true;
						$char = $current;
					}
				}
			}
			if (($current == ';' && !$open) || $i == $end - 1)
			{
				$queries[] = substr($sql, $start, ($i - $start + 1));
				$start = $i + 1;
			}
		}
		return $queries;
	}
}
